/*
 * Flashbook GUI
 * Copyright (c) 2008 Benjamin Moody
 *
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _FB_TEXT_BUFFER_H
#define _FB_TEXT_BUFFER_H

/* FBTextBuffer -- a version of GtkTextBuffer with an undo/redo history. */

#define FB_TYPE_TEXT_BUFFER            (fb_text_buffer_get_type())
#define FB_TEXT_BUFFER(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), FB_TYPE_TEXT_BUFFER, FBTextBuffer))
#define FB_TEXT_BUFFER_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass), FB_TYPE_TEXT_BUFFER, FBTextBufferClass))
#define FB_IS_TEXT_BUFFER(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FB_TYPE_TEXT_BUFFER))
#define FB_IS_TEXT_BUFFER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FB_TYPE_TEXT_BUFFER))
#define FB_TEXT_BUFFER_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), FB_TYPE_TEXT_BUFFER, FBTextBufferClass))


typedef struct _FBTextBuffer FBTextBuffer;
typedef struct _FBTextBufferClass FBTextBufferClass;
typedef struct _FBUndoAction FBUndoAction;

struct _FBTextBuffer {
  GtkTextBuffer parent;

  gint max_chars_per_undo;

  gboolean recording;

  FBUndoAction* undo_stack;
  FBUndoAction* redo_stack;
};

struct _FBTextBufferClass {
  GtkTextBufferClass parent;

  void (* undo_state_changed) (FBTextBuffer *buffer);
  void (* clear_history) (FBTextBuffer *buffer);
  void (* undo) (FBTextBuffer *buffer);
  void (* redo) (FBTextBuffer *buffer);
};

GType fb_text_buffer_get_type(void) G_GNUC_CONST;

GtkTextBuffer* fb_text_buffer_new(void);
void fb_text_buffer_set_recording(FBTextBuffer* buffer,
				  gboolean recording);
void fb_text_buffer_clear_history(FBTextBuffer* buffer);
gboolean fb_text_buffer_can_undo(FBTextBuffer* buffer);
gboolean fb_text_buffer_can_redo(FBTextBuffer* buffer);
void fb_text_buffer_undo(FBTextBuffer* buffer);
void fb_text_buffer_redo(FBTextBuffer* buffer);

#endif
