/*
 * libflashbook - Library for creating eBooks for the TI-83/84 Plus
 * Copyright (c) 2008 Benjamin Moody
 *
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include "fb.h"

/**************** Filename quoting ****************/

#define IS_DANGEROUS(c) ((c) <= ' ' || (c) == '%' || (c) == '<'         \
                         || (c) == '>' || (c) == '&' || (c) == '"'      \
                         || (c) == '\'' || (c) >= 0x7f)

#define TOHEX(a) ((a) > 9 ? (a) - 10 + 'A' : (a) + '0')

/* Escape characters in the given filename */
char* fb_escape_filename(const char* fname, int len)
{
  char* p;
  int i, n;

  if (len < 0)
    len = strlen(fname);

  n = 0;
  for (i = 0; i < len; i++) {
    if (IS_DANGEROUS(fname[i]))
      n += 3;
    else
      n++;
  }

  p = fbnew(char, n + 1);

  n = 0;
  for (i = 0; i < len; i++) {
    if (IS_DANGEROUS(fname[i])) {
      p[n++] = '%';
      p[n++] = TOHEX((fname[i] >> 4) & 0xf);
      p[n++] = TOHEX(fname[i] & 0xf);
    }
    else {
      p[n++] = fname[i];
    }
  }
  p[n] = 0;

  return p;
}

static int from_hex(int c)
{
  if (c >= 'A' && c <= 'F')
    return c + 10 - 'A';
  else if (c >= '0' && c <= '9')
    return c - '0';
  else if (c >= 'a' && c <= 'f')
    return c + 10 - 'a';
  else
    return -1;
}

/* Convert a string previously written by fb_escape_filename(). */
char* fb_unescape_filename(const char* ename, int len)
{
  char* p;
  int i, n;
  int a, b;

  if (len < 0)
    len = strlen(ename);

  n = 0;
  for (i = 0; i < len; n++) {
    if (ename[i] == '%' && i + 2 < len)
      i += 3;
    else
      i++;
  }

  p = fbnew(char, n + 1);

  n = 0;
  for (i = 0; i < len; n++) {
    if (ename[i] == '%' && i + 2 < len) {
      a = from_hex(ename[i + 1]);
      b = from_hex(ename[i + 2]);
      if (a == -1 || b == -1)
        p[n] = '?';
      else
        p[n] = ((a << 4) | b);
      i += 3;
    }
    else {
      p[n] = ename[i++];
    }
  }
  p[n] = 0;

  return p;
}
